/* =====================================================================================
                      The UQ Toolkit (UQTk) version 2.1
                     Copyright (2013) Sandia Corporation
                      http://www.sandia.gov/UQToolkit/

     Copyright (2013) Sandia Corporation. Under the terms of Contract DE-AC04-94AL85000
     with Sandia Corporation, the U.S. Government retains certain rights in this software.

     This file is part of The UQ Toolkit (UQTk)

     UQTk is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
     the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.

     UQTk is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU Lesser General Public License for more details.
 
     You should have received a copy of the GNU Lesser General Public License
     along with UQTk.  If not, see <http://www.gnu.org/licenses/>.
 
     Questions? Contact Bert Debusschere <bjdebus@sandia.gov>
     Sandia National Laboratories, Livermore, CA, USA
===================================================================================== */
/// \file Array2D.h
#ifndef ARRAY2D_H_SEEN
#define ARRAY2D_H_SEEN

#include <stddef.h>
#include <cstdio>
#include <vector>
#include <iostream>
#include <fstream>
#include <iterator>
#include <algorithm>
#include "Array1D.h"

using namespace std;

/// \class  Array2D
/// \brief  Stores data of any type T in a 2D array
///
/// This class also provides a Fortran-like access operator ()
/// as well as a function to access the data in the array through a pointer that
/// can be passed to F77 or C routines.
/// \author  Bert Debusschere <bjdebus@sandia.gov>
/// \date  Jan 2005
/// \note  Inpired by Helgi Adalsteinsson's Array class implementation
/// \todo  Define copy constructor
template <typename T>
class Array2D {
  public:
    /// \brief Default constructor, which does not allocate any memory
    Array2D(): xsize_(0), ysize_(0) {};

    /// \brief Constructor that allocates the memory
    Array2D(const size_t& nx, const size_t& ny):  xsize_(nx), ysize_(ny) {
    data_.resize(xsize_*ysize_);
    }

    /// \brief Constructor that allocates and initializes the data to a constant t
    Array2D(const size_t& nx, const size_t& ny, const T& t):  xsize_(nx), ysize_(ny) {
    data_.resize(xsize_*ysize_ , t);
    }

    /// \brief Copy constructor
    Array2D(const Array2D &obj): xsize_(obj.xsize_), ysize_(obj.ysize_), data_(obj.data_) {};
    
      /// \brief Destructor that frees up the memory
    ~Array2D() {data_.clear();}

    /// \brief Function to clear the memory
    void Clear() {
      xsize_ = 0;
      ysize_ = 0;
      data_.clear();
    }

    /// \brief Returns size in the x-direction
    size_t XSize() const {return xsize_;}
    /// \brief Returns size in the y-direction
    size_t YSize() const {return ysize_;}

    /// \brief Resizes the array
    /// \warning In its current implementation, most of the original data
    /// will get lost if the xsize changes as this changes the indexing for all entries.
    /// \todo Write a better implementation that preserves the original data by
    /// copying it to a temporary array and putting the elements back where they were before.
    /// This would bring this resize() command more closely in line with vector::resize()
    /// function in the original vector class.
    void Resize(const size_t& nx, const size_t& ny) {
      xsize_ = nx;
      ysize_ = ny;
      data_.resize(xsize_*ysize_);
    }

    /// \brief Resizes the array and sets ALL entries to the specified value
    /// \warning All original data will get lost if this function is used!
    /// \todo Write an implementation that is more closely follows the resize
    /// command in the vector class, which keeps the original elements and only
    /// initializes the new elements.
    void Resize(const size_t& nx, const size_t& ny, const T& t) {
      data_.clear();
      xsize_ = nx;
      ysize_ = ny;
      data_.resize(xsize_*ysize_, t);
    }

    /// \brief Set all values in the array to the given value
    void SetValue(const T& t){
      for(size_t i=0; i < data_.size(); i++){
        data_[i] = t;
      }
    }

    /// \brief Return a pointer to the first element of the data in the
    /// vector so we can use it access the data in array format (e.g. for
    /// passing it to a Fortran program).
    T* GetArrayPointer() {
      return &(data_[0]);
    }

    /// \brief Return a cont point to the first element of the data in the
    /// vector so we can use it access the data in array format (e.g. for
    /// passing it to a Fortran program).
    const T* GetConstArrayPointer() const {
      return &(data_[0]);
    }

    /// \brief Fortran-like () operator to access values in the 2D data array
    ///
    /// If "my_data" is an object of type Array2D, then its array values can
    /// be accessed as my_data(ix,iy), where ix, and iy are the indices in the
    /// x, and y dimensions respectively.
    T& operator()(size_t ix,size_t iy) {return data_[ix+xsize_*iy];}

    /// \brief Fortran-like () const operator to access values in the 2D data array
    ///
    /// If "my_data" is an object of type Array2D, then its array values can
    /// be accessed as my_data(ix,iy), where ix, and iy are the indices in the
    /// x, and y dimensions respectively.
    const T& operator()(size_t ix,size_t iy) const {return data_[ix+xsize_*iy];}
   
  private:

    /// \brief Copy constructor, which is made private so it would not be used inadvertently
    /// (until we define a proper copy constructor)
    //Array2D(const Array2D &obj) {};

    /// \brief Number of elements in the x-dimension
    size_t xsize_;
    /// \brief Number of elements in the y-dimension
    size_t ysize_;

    /// \brief Data in the array with size = xsize_ * ysize_
    ///
    /// The data is stored with the fastest running index in the x-dimension
    /// and the slowest running index in the y-dimension. The indices in every dimension
    /// run from 0 to their respective "size-1"
    vector<T> data_;
};

#endif /* ARRAY2D_H_SEEN */
