!
! This version of DALEC is a modified version of the DALEC v1 model [W05,F09].   
! The model has been modified to facilitate comparisons with the
! Community Land Model [T07], and with the Local Terrestrial Ecosystem
! Carbon Model [R11]. 
!
! [W05] Williams et al, doi:10.1111/j.1365-2486.2004.00891.x, (2005).
! [F09] Fox et al, doi:10.1016/j.agrformet.2009.05.002, (2009).
! [T07] Thornton et al, doi:10.1029/2006GB002868, (2007).
! [R11] Ricciuto et al, doi:10.1029/2010JG001400, (2011).
!
module obsdrivers

 ! meteorological drivers
 double precision :: maxt,mint,rad,day,ca,doy,lat,vpd
 double precision, parameter ::  psid=-2.0  !water potential gradient
 double precision, parameter ::  rtot=1.0   !hydraulic resistance
 logical decid

end module obsdrivers

module parameters

 ! parameters in the modified DALEC model
 double precision gdd_min, gdd_max, laimax, tsmin, leaffall
 double precision nue, rg_frac, br_mr, q10_mr
 double precision astem, tleaf, troot, tstem
 double precision q10_hr, br_lit, br_som, dr, lma, leafcn

end module parameters

module moresetup

  integer, dimension(17) :: &
    dayy=(/0,365,731,1096,1461,1826,2192,2557,2922,3287,3653,4018,4383,4748,5114,5479,5844/)
  integer, parameter :: ncsv    = 1  ! no of saved cycles
  integer :: yid

end module moresetup

subroutine dalec_deciduous(nday,ncycles,params,driver,model)

  use obsdrivers
  use parameters
  use moresetup
  implicit none
  integer nday, ncycles
  double precision :: driver(5,*), model(6,*), params(*)
  ! model states
  double precision :: LAI, litfall(2), veg_pools(3)
  double precision :: soil_pools(2), GDD, RG, RM, NPP, NPP2
  double precision :: CF_DELTA, GPP, HR, LAI_last

#ifdef SAVEAVG
  double precision :: modelavg(6)
#endif
  integer :: i, n, isv

  ! set parameters
  astem    = params(1) 
  br_mr    = params(2) 
  br_lit   = params(3) 
  br_som   = params(4) 
  dr       = params(5) 
  gdd_min  = params(6) 
  gdd_max  = params(7) 
  laimax   = params(8) 
  leafcn   = params(9) 
  leaffall = params(10)
  lma      = params(11)
  nue      = params(12)
  q10_mr   = params(13)
  q10_hr   = params(14)
  rg_frac  = params(15)
  tleaf    = params(16) 
  troot    = params(17)
  tstem    = params(18)
  tsmin    = params(19)

#ifdef USEPOOLS
  veg_pools(1)  = params(20)
  veg_pools(2)  = params(21)
  veg_pools(3)  = params(22)
  soil_pools(1) = params(23)
  soil_pools(2) = params(24)
#else
  veg_pools  = 0.d0
  soil_pools = 0.d0
#endif

  ! more input
#ifndef HA1
  lat  = 45.0d0   ! latitude
#else
  lat  = 42.5d0   ! latitude for Harvard forest
#endif
  lat  = lat*3.14159265358979d0/180.d0 ! convert latitude to radians

  LAI        = 0.d0
  isv = 0

  do n = 1,ncycles
    yid = 1
    do i=1,nday
      ! Read in drivers for day
#ifndef HA1
      doy   = MOD(i-1,365)+1 ! driver(1,i)  ! day of year
#else
      ! day of year specific for Harvard forest data
      if ( i > dayy(yid+1) ) then
        yid = yid + 1
      end if
      doy = i-dayy(yid)
#endif
      mint  = driver(2,i)    ! minimum temperature (C)
      maxt  = driver(3,i)    ! maximum temperature (C)
      rad   = driver(4,i)    ! global radiation (MJ m-2 d-1)
      vpd   = 1.0d0 !driver(5,i) ! vapor pressure deficit
      Ca    = driver(5,i)    ! carbon dioxide concentration
      decid = .true.         ! deciduous phenology

      if (doy .eq. 1) GDD=0.0d0  !reset GDD on DOY 1
      GDD=GDD+max(0.5d0*(maxt+mint)-10.0d0,0.0d0) ! growing degree day heat sum

      ! calculate deciduous phenology
      if (decid) then 
        LAI_last = LAI
        call phenology(GDD, LAI)
        CF_DELTA = (LAI-LAI_LAST)*lma  ! change in leaf mass
      else
        LAI = laimax
        CF_DELTA = tleaf*VEG_POOLS(1)
      end if

      ! get gross primary productivity
      if (LAI .gt. 0) then 
        call Phot(LAI, GPP)
      else
        GPP=0.0d0
      end if

      ! get autotrophic respiration
      call Ra(VEG_POOLS, CF_DELTA, GPP, RG, RM)
      NPP  = GPP-RG-RM
      NPP2 = NPP
      if (CF_DELTA .gt. 0) NPP2 = NPP-CF_DELTA

      ! allocate carbon 
      call allocate(CF_DELTA, VEG_POOLS, NPP2)

      ! litter fall
      call litterfall(CF_DELTA, VEG_POOLS, SOIL_POOLS, LITFALL)

      ! decomposition
      call decomp(SOIL_POOLS, HR)

      ! only retain last 'ncsv' cycles 
      if ( n > ncycles-ncsv) then
        isv = isv+1
        model(1,isv) = HR-NPP  ! NEE
        model(2,isv) = GPP     ! GPP
        model(3,isv) = HR      ! LAI
        ! for predictions
        model(4,isv) = LAI
        model(5,isv) = VEG_POOLS(1)+VEG_POOLS(2)+VEG_POOLS(3)
        model(6,isv) = SOIL_POOLS(1)+SOIL_POOLS(2)
      end if
      !write(*,*)model(:,isv)
    end do
  end do 

#ifdef PRINTPOOLS
  write(*,*)VEG_POOLS(1),VEG_POOLS(2),VEG_POOLS(3)
  write(*,*)SOIL_POOLS(1),SOIL_POOLS(2)
#endif

#ifdef SAVEAVG
  ! average data
  modelavg=0.0
  do i = 1,isv
    modelavg = modelavg+model(:,i)
  end do
  open(26, file='./model_output.txt',access='append')
  write(26,'(6(E20.12,1x))') modelavg(1:6)/dble(isv)
  close(26)
#endif

end subroutine DALEC_deciduous

!------------------------------------------------------------------
subroutine Ra(VEG_POOLS, CF_DELTA, GPP, RG, RM)
  ! parameters rg_frac, br_mr, q10_mr
  use obsdrivers
  use parameters
  implicit none
  double precision :: VEG_POOLS(3), GPP, CF_DELTA

  ! Cosmin 20130308 modified Rg(3) into RG
  double precision :: Trate, RG, RM

  ! calculate Q10 temperature factor for maintenace respiration
  ! Cosmin 20130308 commenting the line below since q10_mr is defined as a parameter
  !q10_mr = 2.0d0
  Trate  = q10_mr**((0.5d0*(maxt+mint)-10.d0)/10.d0) 
  ! calculate maintenance respiration
  RM = (VEG_POOLS(1)+VEG_POOLS(2)+VEG_POOLS(3))*br_mr*Trate

  ! calculate growth respiration 
  RG=0.0d0
  if (GPP-RM .gt. 0.0d0) then 
    RG = rg_frac*(GPP-RM)
  end if

  ! growth respiration to force growing leaves if GPP exceeded
  if ( (CF_DELTA .gt. GPP-RM) .and. (CF_DELTA .gt. 0.d0)) then 
    RG = RG+(CF_DELTA-max(GPP-RM,0.0d0))*rg_frac
  end if

end subroutine Ra

!------------------------------------------------------------------
subroutine allocate(CF_DELTA, VEG_POOLS, NPP2)

  ! parameters astem
  use obsdrivers
  use parameters
  implicit none
  double precision :: VEG_POOLS(3), NPP2, CF_DELTA

  ! allocate carbon to vegetation pools
  if (decid) VEG_POOLS(1) = VEG_POOLS(1)+CF_DELTA ! leaf
  VEG_POOLS(2) = VEG_POOLS(2)+astem*NPP2          ! stem
  VEG_POOLS(3) = VEG_POOLS(3)+(1-astem)*NPP2      ! root

end subroutine allocate

!------------------------------------------------------------------
subroutine phenology(GDD, LAI)

  ! parameters gdd_min, gdd_max, laimax, tsmin, leaffall 
  use obsdrivers
  use parameters
  implicit none
  double precision :: GDD, LAI

  ! if spring and GDD requirements met, grow leaves
  if (doy .le. 200) then 
    if (GDD .lt. gdd_min) then 
      LAI = 0.0d0
    else if (GDD .ge. gdd_min .and. GDD .lt. gdd_max) then 
      LAI = laimax*(GDD-gdd_min)/(gdd_max-gdd_min)
    else
      LAI = laimax
    end if
  else
    ! if autumn and cold, drop leves
    if (mint .lt. tsmin .and. LAI .gt. 0) then 
      LAI = LAI - leaffall*laimax
    end if
    if (LAI .lt. 0.d0) LAI = 0.d0
  end if

end subroutine phenology

!-----------------------------------------------------------------

subroutine litterfall(CF_DELTA, VEG_POOLS, SOIL_POOLS, LITFALL)
  !parameters tleaf, troot, tstem
  use obsdrivers
  use parameters
  implicit none

  double precision :: CF_DELTA, VEG_POOLS(3), LITFALL(2), SOIl_POOLS(2)

  LITFALL(1) = VEG_POOLS(3)*troot
  LITFALL(2) = VEG_POOLS(2)*tstem
  VEG_POOLS(3) = VEG_POOLS(3) - LITFALL(1)
  VEG_POOLS(2) = VEG_POOLS(2) - LITFALL(2)
  if (decid) then 
    if (CF_DELTA .lt. 0) LITFALL(1) = -CF_DELTA
  else
    LITFALL(1) = LITFALL(1) + tleaf * VEG_POOLS(1)
  end if
  SOIL_POOLS(1) = SOIL_POOLS(1)+LITFALL(1)
  SOIL_POOLS(2) = SOIL_POOLS(2)+LITFALL(2)

end subroutine litterfall

!------------------------------------------------------------------
subroutine decomp(SOIL_POOLS, HR)
  !module for computing decomposition
  !parameters q10_hr, br_lit, br_som, dr
  use obsdrivers
  use parameters
  implicit none
  double precision :: soil_pools(2), HR, D, Trate

  !q10_hr=2.0
  Trate=q10_hr**((0.5d0*(maxt+mint)-10.d0)/10.d0) 
  HR = br_lit*SOIL_POOLS(1)*Trate
  SOIL_POOLS(1) = SOIL_POOLS(1) - br_lit*SOIL_POOLS(1)*Trate
  HR = HR + br_som*SOIL_POOLS(2)*Trate
  SOIL_POOLS(2) = SOIL_POOLS(2) - br_som*SOIL_POOLS(2)*Trate
  D = dr*SOIL_POOLS(1)   !decomposition (litter to SOM)
  SOIL_POOLS(1) = SOIL_POOLS(1)-D
  SOIL_POOLS(2) = SOIL_POOLS(2)+D

end subroutine decomp

!-------------------------------------------------------------------
! Aggregated Canopy Model
!-------------------------------------------------------------------
subroutine Phot(LAI, GPP)
  !parameters lma, nue, leafcn
  USE obsdrivers
  USE parameters
  implicit none

  double precision :: LAI, GPP, NIT 
  double precision :: cps,e0,gs,ci,qq,pp,trange,model,dec,mult,dayl,pi,a(10)
  pi = 3.14159265358979d0
  trange=0.5*(maxt-mint)

  ! parameter values
  a( 1) =  nue          !nitrogen use efficiency
  a( 2) =  0.0156935d0
  a( 3) =  4.22273d0
  a( 4) =  208.868d0
  a( 5) =  0.0453194d0
  a( 6) =  0.37836d0
  a( 7) =  7.19298d0
  a( 8) =  0.011136d0
  a( 9) =  2.1001d0
  a(10) =  0.789798d0

  Nit = lma/leafcn
  gs = dabs(psid)**a(10)/((a(6)*rtot+trange))
  pp = LAI*Nit/gs*a(1)*dexp(a(8)*maxt)
  qq = a(3)-a(4)

  !internal co2 concentration
  ci = 0.5d0*(Ca+qq-pp+((Ca+qq-pp)**2-4.d0*(Ca*qq-pp*a(3)))**0.5)
  e0 = a(7)*LAI**2/(LAI**2+a(9))

  dec  = -23.4d0*dcos((360.d0*(doy+10.d0)/365.)*pi/180.d0)*pi/180.d0
  mult = dtan(lat)*dtan(dec)
  if (mult.ge.1.) then
    dayl=24.0d0
  else if (mult.le.-1.) then
    dayl=0.d0
  else
    dayl=24.d0*dacos(-mult)/pi
  end if

  cps   = e0*rad*gs*(Ca-ci)/(e0*rad+gs*(Ca-ci))
  model = cps*(a(2)*dayl+a(5))
  gpp   = model

END subroutine  Phot

